@@ -1,7 +1,7 @@
 
 ########################################
 # This is a copy of msm::dtnorm
-tam_dtnorm <- function (x, mean = 0, sd = 1, lower = -Inf, upper = Inf, log = FALSE) 
+tam_dtnorm <- function(x, mean = 0, sd = 1, lower = -Inf, upper = Inf, log = FALSE) 
 {
     ret <- numeric(length(x))
     ret[x < lower | x > upper] <- if (log) 
@@ -17,5 +17,5 @@ tam_dtnorm <- function (x, mean = 0, sd = 1, lower = -Inf, upper = Inf, log = FA
         else xtmp <- xtmp/denom
         ret[x >= lower & x <= upper] <- xtmp[ind]
     }
-    ret
+    return(ret)
 }
\ No newline at end of file
