@@ -8,8 +8,17 @@ tam_mml_3pl_calc_prob <- function(iIndex, A, AXsi, B, xsi, theta,
            nnodes, maxK, recalc=TRUE , guess)
 {		   
     if(recalc){
-      AXsi.tmp <- array( tensor::tensor( A[iIndex,,, drop = FALSE], xsi, 3, 1 ) , 
-                         dim = c( length(iIndex) , maxK , nnodes ) )
+     # AXsi.tmp <- array( tensor::tensor( A[iIndex,,, drop = FALSE], xsi, 3, 1 ) , 
+     #                    dim = c( length(iIndex) , maxK , nnodes ) )
+						 
+     LI <- length(iIndex)
+	 LXsi <- dim(A)[3]
+     AXsi.tmp <- array( 0 , dim = c( LI , maxK , nnodes ) )
+	 for (kk in 1:maxK){
+	    A_kk <- matrix( A[ iIndex , kk , ] , nrow = LI , ncol = LXsi )
+		AXsi.tmp[, kk , 1:nnodes ] <- A_kk %*% xsi
+	 }		 
+						 
       AXsi[iIndex,] = AXsi.tmp[,,1]
     } else {
       AXsi.tmp <- array( AXsi, dim = c( length(iIndex) , maxK , nnodes ) )
@@ -21,9 +30,15 @@ tam_mml_3pl_calc_prob <- function(iIndex, A, AXsi, B, xsi, theta,
       Btheta <- Btheta + array(B[iIndex,,dd ,drop = FALSE] %o% theta[,dd] , dim = dim(Btheta))
 							}
 							
-    rprobs <- ( rr <- exp(Btheta+AXsi.tmp) )/aperm( array( rep( colSums( aperm( rr , c(2,1,3) ) ,
-					dims=1 , na.rm = TRUE) ,    maxK ), dim=dim(rr)[c(1,3,2)] ) , c(1,3,2) )
-					
+    # rprobs <- ( rr <- exp(Btheta+AXsi.tmp) )/aperm( array( rep( colSums( aperm( rr , c(2,1,3) ) ,
+	#				dims=1 , na.rm = TRUE) ,    maxK ), dim=dim(rr)[c(1,3,2)] ) , c(1,3,2) )
+
+	#*** subtract maximum and use in Rcpp
+	rr0 <- Btheta + AXsi.tmp		
+	rr1 <- tam_calc_prob_helper_subtract_max( rr0=rr0 )
+	rr <- exp(rr1)
+    rprobs <- rr / aperm( array( rep( colSums( aperm( rr , c(2,1,3) ) ,
+					dims=1 , na.rm = TRUE) ,    maxK ), dim=dim(rr)[c(1,3,2)] ) , c(1,3,2) )							
 	# include guessing	
 	rprobs0 <- rprobs
 	ind <- which(guess[ iIndex ] > 1E-6 )
